! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
!
!+
! Module: sppartial_mod
!
! Authors:  Steven L. Snowden - Goddard Space Flight Center (NASA)
!
! Description: Extended Source Analysis Software
!
! Comments:
!
! $Id: sppartial_mod.f90,v 1.2 2009/02/20 13:27:08 aibarra Exp $
!-
!
!  Version 5.0: 7 November 2012
!
!  Returns the average SP flux and BACKSCAL keyword for a region
!
!  sppartial uses the results from a spectral fit of a limited
!  region of the FOV to better determine the contribution of
!  SP contamination.  This is to avoid the effect of a bright
!  extended region skewing the SP component fit.  It uses the
!  SP templates from the limited region and the full region plus
!  the fitted SP normalization from the limited region to find
!  the SP normalization for the full FOV.
!
!  Input:
!    CalDB data directory
!    MOS number
!    full region image file name
!    full region spectrum file name
!    selected region image file name
!    selected region spectrum file name
!    selected region SP normalization
!
!    Example: sp-partial /path/qpb 1 full-image.fits full-spec.fits
!                   sel-image.fits sel-spec.fits norm
!
! Usage:
!   data directory
!   MOS number
!   full region image file name
!   full region spectrum file name
!   selected region image file name
!   selected region spectrum file name
!   selected region SP normalization
! 
!   Example: 
!     sp-partial /path/qpb 1 full-image.fits 
!                     full-spec.fits sel-image.fits 
!                     sel-spec.fits norm

module sppartial_mod

  use cal
  use dal
  use param 
  use errorHandling
  use gofutils_mod  ! from goflib, formerly in marq_mod
  use gofio_mod     ! from goflib, formerly in subsn (SS)

  implicit none

  private

  character(len=*), parameter :: &
       fimagepar = "fullimage", &  ! full region image filename
       fspecpar  = "fullspec", &   ! full region spectral filename
       rimagepar = "regimage", &   ! selected region image filename
       rspecpar  = "regspec", &    ! selected region spectral filename
       rnormpar  = "regnorm"       ! selected region Soft Proton normalization
  
  public sppartial

contains

  !---------------------------------------------------------------------
  subroutine sppartial
    !---------------------------------------------------------------------
    type(DataSetT) :: fimage_S, rimage_S, fspec_S, rspec_S
    type(TableT)   :: fimage_T, rimage_T, fspec_T, rspec_T
    type(ArrayT)   :: fimage_A, rimage_A, fspec_A, rspec_A
    character(len=2) :: instFI, instFS, instRI, instRS
    character(len=5) :: expid
    character(len=6) :: filterFI, filterFS, filterRI, filterRS 
    character(len=10) :: obsidFI, obsidFS, obsidRI, obsidRS
    character(len=25) :: smode
    character(len=256) :: fimage_F, rimage_F, fspec_F, rspec_F
    integer :: status, typeF, mosnum, elow, ehigh, nRows, i, ii
    integer(kind=int16), dimension(:,:), pointer :: aDI16                                
    integer(kind=int32), dimension(:,:), pointer :: aDI32, fimage_IP, rimage_IP
    integer :: frmtime, revolut, n
    integer :: fimgXsize, fimgYsize, rimgXsize, rimgYsize
    logical :: warndie
    real :: backFS, backRS, back, area, expo, spfull, spregion
    real(kind=single), dimension(:,:), pointer :: aDR32
    real(kind=single), dimension(:,:), pointer :: spdetmap_RP
    real(kind=double), dimension(:,:), pointer :: aDR64                                       
    real(kind=double) :: rnorm, rrnorm

    fimage_F = stringParameter(fimagepar)
    fspec_F  = stringParameter(fspecpar)
    rimage_F = stringParameter(rimagepar)
    rspec_F  = stringParameter(rspecpar)
    rnorm    = realParameter(rnormpar)
    !
    ! Read in full image file
    !
    write(errstr,*) "Full image file: ",trim(adjustl(fimage_F))
    call message(AppMsg,Always)
    status = 0
    warndie = .TRUE.
    call check_open_set(trim(adjustl(fimage_F)),fimage_S,1,status,warndie)
    warndie = .TRUE.     ! see read_header_info in gofio_mod for behavior                
    typeF = 3 ! 1 = evli, 2 = spec, 3 = image                                            
    call read_header_info(typeF,fimage_S,fimage_T,fimage_A,instFI,obsidFI,&             
         expid,smode,frmtime,filterFI,revolut,back,area,expo,elow,&               
         ehigh,nRows,warndie)
    call read_2Darray(fimage_S,fimage_A,aDR32,aDR64,aDI16,fimage_IP,&              
         fimgXsize,fimgYsize)
    call release(fimage_S)
    !
    filterFI = to_upper(filterFI)
    !
    ! Read in full spectral file
    !
    write(errstr,*) "Full spec file: ",trim(adjustl(fspec_F))
    call message(AppMsg,Always)
    status = 0
    warndie = .TRUE.
    call check_open_set(trim(adjustl(fspec_F)),fspec_S,1,status,warndie)
    warndie = .TRUE.     ! see read_header_info in gofio_mod for behavior                
    typeF = 2 ! 1 = evli, 2 = spec, 3 = image                                            
    call read_header_info(typeF,fspec_S,fspec_T,fspec_A,instFS,obsidFS,&             
         expid,smode,frmtime,filterFS,revolut,backFS,area,expo,elow,&               
         ehigh,nRows,warndie)
    ! only need backscale (backFS), so close full spectral file
    call release (fspec_S)
    filterFS = to_upper(filterFS)        
    backFS = backFS / 1440000.0
    if (trim(adjustl(obsidFI)) .ne. trim(adjustl(obsidFS))) then 
       call error("notSAMEobsid","Full image and Full spectrum have different obsids.")
    end if
    if (trim(adjustl(filterFI)) .ne. trim(adjustl(filterFS))) then 
       call error("notSAMEfilter","Full image and Full spectrum have different filters.")
    end if
    !
    ! get the SPDETMAP from the CAL
    !
    call CAL_setState(fimage_F)
    call CAL_spdetmap(spdetmap_RP)
    !
    !  Loop over the image
    !
    n = 0
    spfull = 0.
    do i=0,fimgXsize-1
       do ii=0,fimgYsize-1
          if (fimage_IP(i,ii) .gt. 0) then
             n = n + 1
             spfull = spfull + spdetmap_RP(i,ii)
          endif
       enddo
    enddo
    spfull = spfull / real(n)
    !    
    ! Repeat for the selected region
    !
    ! Read in region image file
    !
    write(errstr,*) "Region image file: ",trim(adjustl(rimage_F))
    call message(AppMsg,Always)
    status = 0
    warndie = .TRUE.
    call check_open_set(trim(adjustl(rimage_F)),rimage_S,1,status,warndie)
    warndie = .TRUE.     ! see read_header_info in gofio_mod for behavior                
    typeF = 3 ! 1 = evli, 2 = spec, 3 = image                                            
    call read_header_info(typeF,rimage_S,rimage_T,rimage_A,instRI,obsidRI,&             
         expid,smode,frmtime,filterRI,revolut,back,area,expo,elow,&               
         ehigh,nRows,warndie)
    call read_2Darray(rimage_S,rimage_A,aDR32,aDR64,aDI16,rimage_IP,&              
         rimgXsize,rimgYsize)
    call release(rimage_S)
    !
    filterRI = to_upper(filterRI)
    !
    ! Read in region spectral file
    !
    write(errstr,*) "Region spec file: ",trim(adjustl(rspec_F))
    call message(AppMsg,Always)
    status = 0
    warndie = .TRUE.
    call check_open_set(trim(adjustl(rspec_F)),rspec_S,1,status,warndie)
    warndie = .TRUE.     ! see read_header_info in gofio_mod for behavior                
    typeF = 2 ! 1 = evli, 2 = spec, 3 = image                                            
    call read_header_info(typeF,rspec_S,rspec_T,rspec_A,instRS,obsidRS,&             
         expid,smode,frmtime,filterRS,revolut,backRS,area,expo,elow,&               
         ehigh,nRows,warndie)
    ! only need backscale (backFS), so close full spectral file
    call release (rspec_S)
    filterRS = to_upper(filterRS)        
    backRS = backRS / 1440000.0
    if (trim(adjustl(obsidRI)) .ne. trim(adjustl(obsidRS))) then 
       call error("notSAMEobsid","Region image and region spectrum have different obsids.")
    end if
    if (trim(adjustl(filterFI)) .ne. trim(adjustl(filterFS))) then 
       call error("notSAMEfilter","Region image and region spectrum have different filters.")
    end if
    !
    ! get the SPDETMAP from the CAL
    !
    call CAL_setState(rimage_F)
    call CAL_spdetmap(spdetmap_RP)
    !
    !  Loop over the image
    ! put in checks for imgXsize/imgYsize!!!???
    n = 0
    spregion = 0.
    do i=0,rimgXsize-1
       do ii=0,rimgYsize-1
          if (rimage_IP(i,ii) .gt. 0) then
             n = n + 1
             spregion = spregion + spdetmap_RP(i,ii)
          endif
       enddo
    enddo
    spregion = spregion / real(n)
    !
    !  Determine the old normalization
    !
    rrnorm = rnorm * (backFS/backRS) * (spfull/spregion)
    !
    write(errstr,*) "Full spectrum backscale  : ",backFS
    call message(AppMsg,Always)
    write(errstr,*) "Region spectrum backscale: ",backRS
    call message(AppMsg,Always)
    write(errstr,*) "Selected Normalization: ",rnorm
    call message(AppMsg,Always)
    write(errstr,*) "Scaled Normalization  : ",rrnorm
    call message(AppMsg,Always)
    
    return

  end subroutine sppartial

end module sppartial_mod
